#! /bin/bash
#
# Author: Markus Gruenwald
# Url: www.mg-it.net
# Date: 2013-10-01
#
#
#


PROGNAME="${0##*/}"
PROGPATH="${0%/*}"
VERSION="20131023"

RUNNING_AS=$(whoami)

. $PROGPATH/utils.sh



NAGIOS_PATH="/usr/local/nagios"



print_usage() {
    echo ""
    echo "Usage: $PROGNAME -H <IP-address> -C <snmp community string> -e <check_name>"

    echo ""
    echo ""
    echo "Options:"
    echo ""
    echo " -H <IP-address>"
    echo ""
    echo " -C <snmp community string>"
    echo ""
    echo " -e <check_name>"
    echo ""
    echo "    available checks for execute:"
    echo ""
    echo "    [system_info]"
    echo "       "
    echo ""
    echo ""
    echo "    [cluster_status]"
    echo "       "
    echo ""
    echo ""
    echo "    [session_stats]"
    echo "       "
    echo ""
    echo ""
    echo "    [ha_statistics]"
    echo "       "
    echo ""
    echo ""
    echo "    [interface] [name]"
    echo "       -e interface 1/3"
    echo ""
    echo ""
    echo "    [hw_health]"
    echo "       gibts noch nicht"
    echo ""
    echo ""
    echo "    [sys_usage]"
    echo "       gibts noch nicht"
    echo ""
    echo ""
    echo "    --help"
    echo "    --version"
    echo ""
}


print_help() {
    print_head
    echo ""
    print_usage
    echo ""
    echo "What is this? Plugin for Nagios?"
    echo ""
}


print_head() {
	echo "$PROGNAME v$VERSION"
       echo -e "This plugin come with ABSOLUTELY NO WARRANTY. You may redistribute\ncopies of the plugins under the terms of the GNU General Public License.\n"
}


###########
# http://support.citrix.com/article/CTX132381

## MIB-II OIDs
# system (1.3.6.1.2.1.1)
# sysDescr (1.3.6.1.2.1.1.1)
# sysUpTime (1.3.6.1.2.1.1.3)
#
## NetScaler Enterprise OIDs
# nsSysGroup (1.3.6.1.4.1.5951.4.1.1) - This provides information about the system properties of Netscaler product. 
# sysBuildVersion (1.3.6.1.4.1.5951.4.1.1.1)
# sysIpAddress (1.3.6.1.4.1.5951.4.1.1.2)
# sysNetmask (1.3.6.1.4.1.5951.4.1.1.3)
# sysHighAvailabilityMode (1.3.6.1.4.1.5951.4.1.1.6)
# sysHardwareVersionId (1.3.6.1.4.1.5951.4.1.1.10)
# sysHardwareVersionDesc (1.3.6.1.4.1.5951.4.1.1.11) 
# sysHardwareEncodedSerialNumber (1.3.6.1.4.1.5951.4.1.1.15) 

check_system_info() {

  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR

  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then
      SYS_UPTIME=$($SNMPGET -v1 -O v $HOSTADDR_1 -c $COMMUNITY SNMPv2-MIB::sysUpTime.0)
      ((SEC=$(echo $SYS_UPTIME |cut -d "(" -f2|cut -d ")" -f1)/100))
      ((BOOT_TIMESTAMP=$(date +%s)-$SEC))
      BOOT_DATE=$(date -d @$BOOT_TIMESTAMP +%c)
      SYS_UPTIME=$(echo $SYS_UPTIME|cut -d")" -f2|cut -d "." -f1)
      SYS_DESCR=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.2.1.1.1.0)
      HW_SN=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.15.0)
      SNMP_OUT=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.2.0 .1.3.6.1.4.1.5951.4.1.1.3.0 .1.3.6.1.4.1.5951.4.1.1.6.0 .1.3.6.1.4.1.5951.4.1.1.10.0 .1.3.6.1.4.1.5951.4.1.1.11.0)

      read SYS_IP SYS_MASK  HA_MODE HW_ID HW_VERSION <<< $SNMP_OUT

      RETSTR="<p>PING: $ICMP_OUT<br /></p>$SYS_DESCR<br />Uptime: $SYS_UPTIME ($BOOT_DATE)<br />Management-IP: $SYS_IP/$SYS_MASK<br />HA-Mode: <b>$HA_MODE</b><br />Hardware ID: $HW_ID<br />Hardware Version: $HW_VERSION<br />Serial: $HW_SN|$ICMP_PERFDATA"
      RETVAL=$STATE_OK

    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>$SYS_DESCR<br />Uptime: $BOOT_DATE / $SYS_UPTIME<br />Management-IP: $SYS_IP/$SYS_MASK<br />HA-Mode: <b>$HA_MODE<br />Hardware ID: $HW_ID<br />Hardware Version: $HW_VERSION<br />Serial: $HW_SN|$ICMP_PERFDATA"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
      RETVAL=STATE_UNKNOWN
  fi

}

## MIB-II OIDs
# ifTable (1.3.6.1.2.1.2.2)
# ifIndex (1.3.6.1.2.1.2.2.1.1)
# ifDescr (1.3.6.1.2.1.2.2.1.2)
# ifType (1.3.6.1.2.1.2.2.1.3)
# ifMtu (1.3.6.1.2.1.2.2.1.4)
# ifSpeed (1.3.6.1.2.1.2.2.1.5)
# ifPhysAddress (1.3.6.1.2.1.2.2.1.6)
# ifAdminStatus (1.3.6.1.2.1.2.2.1.7)
# ifOperStatus (1.3.6.1.2.1.2.2.1.8)
# ifLastChange (1.3.6.1.2.1.2.2.1.9)
# ifInOctets (1.3.6.1.2.1.2.2.1.10)
# ifInUcastPkts (1.3.6.1.2.1.2.2.1.11)
# ifInNUcastPkts (1.3.6.1.2.1.2.2.1.12)
# ifInDiscards (1.3.6.1.2.1.2.2.1.13)
# ifInErrors (1.3.6.1.2.1.2.2.1.14)
# ifInUnknownProtos (1.3.6.1.2.1.2.2.1.15)
# ifOutOctets (1.3.6.1.2.1.2.2.1.16)
# ifOutUcastPkts (1.3.6.1.2.1.2.2.1.17)
# ifOutNUcastPkts (1.3.6.1.2.1.2.2.1.18)
# ifOutDiscards (1.3.6.1.2.1.2.2.1.19)
# ifOutErrors (1.3.6.1.2.1.2.2.1.20)
# ifOutQLen (1.3.6.1.2.1.2.2.1.21)
# ifSpecific (1.3.6.1.2.1.2.2.1.22) 

check_interface() {

  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR

  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then

      OID_BASE=".1.3.6.1.4.1.5951.4.1.1.54.1"
      OID_IF_NAME=".1.3"
      OID_IF_ALIAS=".52.3"
      OID_IF_MEDIA=".2.3"
      OID_IF_TOTRXBYTES=".3.3"
      OID_IF_RXAVGBANDWIDTHUSAGE=".4.3"
      OID_IF_TOTRXPKTS=".5.3"
      OID_IF_RXAVGPACKETRATE=".6.3"
      OID_IF_RXCRCERRORS=".11.3"
      OID_IF_ERRDROPPEDRXPKTS=".22.3"
      OID_IF_TOTTXBYTES=".7.3"
      OID_IF_TXAVGBANDWIDTHUSAGE=".8.3"
      OID_IF_TOTTXPKTS=".9.3"
      OID_IF_TXAVGPACKETRATE=".10.3"
      OID_IF_TXCRCERRORS=".11.3"
      OID_IF_ERRDROPPEDTXPKTS=".38.3"
      # IF-MIB::ifInMulticastPkts.$IF_MIB_ID
      # IF-MIB::ifInBroadcastPkts.$IF_MIB_ID
      # IF-MIB::ifOutMulticastPkts.$IF_MIB_ID
      # IF-MIB::ifOutBroadcastPkts.$IF_MIB_ID

      IF_INDEX_ARR=($($SNMPWALK -v1 -O neq $HOSTADDR_1 -c $COMMUNITY ${OID_BASE}${OID_IF_NAME}|tr -d '"'|tr " " ";"|sed 's/'${OID_BASE}${OID_IF_NAME}'//g'))
      IF_MIB_INDEX=($($SNMPWALK -v1 -O neq $HOSTADDR_1 -c $COMMUNITY IF-MIB::ifName|cut -d "." -f13-|tr " " ";"))

      PERFDATA=""
      RETSTR=""
      IF_FOUND=0

      for I in ${IF_INDEX_ARR[*]}
        do
          IF_OID="${I%;*}"
          IF_NAME=${I##*;} # IF_NAME

          [[ "$IF_NAME_CMD" == "$IF_NAME" ]] || continue 

          for I in  ${IF_MIB_INDEX[@]}
            do
              [[ "${I##*;}" == "$IF_NAME" ]] && IF_MIB_ID=${I%;*} && IF_FOUND=1
            done

          MEDIA=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY ${OID_BASE}${OID_IF_MEDIA}${IF_OID})
          SNMP_OUT=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY \
          ${OID_BASE}${OID_IF_ALIAS}${IF_OID} \
          ${OID_BASE}${OID_IF_TOTRXBYTES}${IF_OID} \
          ${OID_BASE}${OID_IF_RXAVGBANDWIDTHUSAGE}${IF_OID} \
          ${OID_BASE}${OID_IF_TOTRXPKTS}${IF_OID} \
          ${OID_BASE}${OID_IF_RXAVGPACKETRATE}${IF_OID} \
          ${OID_BASE}${OID_IF_RXCRCERRORS}${IF_OID} \
          ${OID_BASE}${OID_IF_ERRDROPPEDRXPKTS}${IF_OID} \
          ${OID_BASE}${OID_IF_TOTTXBYTES}${IF_OID} \
          ${OID_BASE}${OID_IF_TXAVGBANDWIDTHUSAGE}${IF_OID} \
          ${OID_BASE}${OID_IF_TOTTXPKTS}${IF_OID} \
          ${OID_BASE}${OID_IF_TXAVGPACKETRATE}${IF_OID} \
          ${OID_BASE}${OID_IF_TXCRCERRORS}${IF_OID} \
          ${OID_BASE}${OID_IF_ERRDROPPEDTXPKTS}${IF_OID} \
          IF-MIB::ifInMulticastPkts.$IF_MIB_ID \
          IF-MIB::ifInBroadcastPkts.$IF_MIB_ID \
          IF-MIB::ifOutMulticastPkts.$IF_MIB_ID \
          IF-MIB::ifOutBroadcastPkts.$IF_MIB_ID \
          )

          read ALIAS TOTRXBYTES RXAVGBANDWIDTHUSAGE TOTRXPKTS RXAVGPACKETRATE RXCRCERRORS ERRDROPPEDRXPKTS TOTTXBYTES TXAVGBANDWIDTHUSAGE TOTTXPKTS TXAVGPACKETRATE TXCRCERRORS ERRDROPPEDTXPKTS INMULTICASTPKTS INBROADCASTPKTS OUTMULTICASTPKTS OUTBROADCASTPKTS <<< $SNMP_OUT

          [[ "$MEDIA" == "\"\"" ]] && MEDIA=down
          [[ "$ALIAS" != "\"\"" ]] && ALIAS="($ALIAS)" || ALIAS=""

          if [[ "$RETSTR" == "" ]]
            then
              [[ "$ALIAS" != "" ]] && \
              RETSTR=" \
                Interface_$IF_NAME $ALIAS<br /> \
                Status: $MEDIA<br /> \
                packets in: $TOTRXPKTS, bytes: $TOTRXBYTES, CRC error: $RXCRCERRORS, err dropped: $ERRDROPPEDRXPKTS<br /> \
                packets out: $TOTTXPKTS, bytes: $TOTTXBYTES, CRC error: $TXCRCERRORS, err dropped: $ERRDROPPEDTXPKTS<br /> \
                multicast in: $INMULTICASTPKTS, out: $OUTMULTICASTPKTS, broadcast in: $INBROADCASTPKTS, out: $OUTBROADCASTPKTS\
                " \
              || \
              RETSTR=" \
                Interface_$IF_NAME $ALIAS<br /> \
                Status: $MEDIA<br />"


          else
            [[ "$ALIAS" != "" ]] && \
              RETSTR="$RETSTR<p></p> \
              Interface_$IF_NAME $ALIAS<br /> \
              Status: $MEDIA<br /> \
              packets in: $TOTRXPKTS, bytes: $TOTRXBYTES, CRC error: $RXCRCERRORS, err dropped: $ERRDROPPEDRXPKTS<br /> \
              packets out: $TOTTXPKTS, bytes: $TOTTXBYTES, CRC error: $TXCRCERRORS, err dropped: $ERRDROPPEDTXPKTS<br /> \
              multicast packets in: $INMULTICASTPKTS, out: $OUTMULTICASTPKTS, broadcast packets in: $INBROADCASTPKTS, out: $OUTBROADCASTPKTS\
              " \
            || \
              RETSTR="$RETSTR<p></p> \
              Interface_$IF_NAME $ALIAS<br /> \
              Status: $MEDIA<br />"
          fi

          IF_NAME=$(echo $IF_NAME|tr "/" ":")
          if [[ "$PERFDATA" == "" ]]
            then
              PERFDATA="|\
              ${IF_NAME}.pkts_in=${TOTRXPKTS}c;;;; \
              ${IF_NAME}.bytes_in=${TOTRXBYTES}c;;;; \
              ${IF_NAME}.crcerr_in=${RXCRCERRORS}c;;;; \
              ${IF_NAME}.errdrop_in=${ERRDROPPEDRXPKTS}c;;;; \
              ${IF_NAME}.pkts_out=${TOTTXPKTS}c;;;; \
              ${IF_NAME}.bytes_out=${TOTTXBYTES}c;;;; \
              ${IF_NAME}.crcerr_out=${TXCRCERRORS}c;;;; \
              ${IF_NAME}.errdrop_out=${ERRDROPPEDTXPKTS}c;;;; \
              ${IF_NAME}.mcpkts_in=${INMULTICASTPKTS}c;;;; \
              ${IF_NAME}.mcpkts_out=${OUTMULTICASTPKTS}c;;;; \
              ${IF_NAME}.bcpkts_in=${INBROADCASTPKTS}c;;;; \
              ${IF_NAME}.bcpkts_out=${OUTBROADCASTPKTS}c;;;; \
              "
          else
            PERFDATA="$PERFDATA \
            ${IF_NAME}.pkts_in=${TOTRXPKTS}c;;;; \
            ${IF_NAME}.bytes_in=${TOTRXBYTES}c;;;; \
            ${IF_NAME}.crcerr_in=${RXCRCERRORS}c;;;; \
            ${IF_NAME}.errdrop_in=${ERRDROPPEDRXPKTS}c;;;; \
            ${IF_NAME}.pkts_out=${TOTTXPKTS}c;;;; \
            ${IF_NAME}.bytes_out=${TOTTXBYTES}c;;;; \
            ${IF_NAME}.crcerr_out=${TXCRCERRORS}c;;;; \
            ${IF_NAME}.errdrop_out=${ERRDROPPEDTXPKTS}c;;;; \
            ${IF_NAME}.mcpkts_in=${INMULTICASTPKTS}c;;;; \
            ${IF_NAME}.mcpkts_out=${OUTMULTICASTPKTS}c;;;; \
            ${IF_NAME}.bcpkts_in=${INBROADCASTPKTS}c;;;; \
            ${IF_NAME}.bcpkts_out=${OUTBROADCASTPKTS}c;;;; \
            "
          fi

        done


      [[ "$IF_FOUND" != "1" ]] && echo "Interface $IF_NAME_CMD not found ..." && exit $STATE_UNKNOWN

    RETSTR=$(echo ${RETSTR}${PERFDATA}|tr -d '"')
      RETVAL=$STATE_OK

    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT"
      RETVAL=STATE_UNKNOWN
  fi

  
  
}


## NetScaler Enterprise OIDs

check_hw_health() {
  echo "exec hw_health check ... Check gibts noch nicht (in Arbeit)"
}


# aaaCurICASessions (1.3.6.1.4.1.5951.4.1.1.67.10)
# Count of current ICA only sessions. 

check_session_stats() {
  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR

  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then
      CURICASESSIONS=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY 1.3.6.1.4.1.5951.4.1.1.67.10.0|tr -d '"')

      PERFDATA="|CurICASessions=${CURICASESSIONS};;;;"
      RETSTR="CurICASessions: ${CURICASESSIONS}${PERFDATA}"
      RETVAL=$STATE_OK

    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
      RETVAL=STATE_UNKNOWN
  fi

}


## NetScaler Enterprise OIDs
# nsCPUTable (1.3.6.1.4.1.5951.4.1.1.41.6) - This table contains information about each CPU in NetScaler. This is indexed on the nsCPUname .
# nsCPUname (1.3.6.1.4.1.5951.4.1.1.41.6.1.1) - The name of the CPU.
# nsCPUusage (1.3.6.1.4.1.5951.4.1.1.41.6.1.2) - CPU utilization percentage. 
#
# nsResourceGroup (1.3.6.1.4.1.5951.4.1.1.41) - This provides information about system resources in the Netscaler product.
# resCpuUsage (1.3.6.1.4.1.5951.4.1.1.41.1) - CPU utilization percentage.
# resMemUsage (1.3.6.1.4.1.5951.4.1.1.41.2) - Percentage of memory utilization on NetScaler.
# numCPUs (1.3.6.1.4.1.5951.4.1.1.41.3) - The number of active CPUs.
# memSizeMB (1.3.6.1.4.1.5951.4.1.1.41.4) - Total amount of system memory, in megabytes. 


check_sys_usage() {
  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR

  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then
      SNMP_OUT=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY 1.3.6.1.4.1.5951.4.1.1.41.1.0 1.3.6.1.4.1.5951.4.1.1.41.2.0)

      read PERC_CPU_USAGE PERC_MEM_USAGE <<< $SNMP_OUT

      PERFDATA="|CpuUsage=${PERC_CPU_USAGE}%;;;; MemUsage=${PERC_MEM_USAGE}%;;;;"
      RETSTR="CpuUsage: ${PERC_CPU_USAGE}%<br />MemUsage: ${PERC_MEM_USAGE}%${PERFDATA}"
      RETVAL=$STATE_OK

    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT|$ICMP_PERFDATA"
      RETVAL=STATE_UNKNOWN
  fi
}


## NetScaler Enterprise OIDs
# nsHighAvailabilityGroup (1.3.6.1.4.1.5951.4.1.1.23) - This provides information about the High Availability feature in the Netscaler product.
# haPeerIpAddress (1.3.6.1.4.1.5951.4.1.1.23.2) - This represents the ipaddress of the failover peer NetScaler(Only for HA over IPv4). For HA over IPv6 (as well as IPv4) haPeerInetAddr will contain this information.
# haPeerState (1.3.6.1.4.1.5951.4.1.1.23.3) - This represents the state of the failover peer NetScaler whether Primary or Secondary
# haTotStateTransitions (1.3.6.1.4.1.5951.4.1.1.23.4 - Total number of master state changes that the NetScaler appliance has made from primary to secondary and vice-versa.
# haTimeofLastStateTransition (1.3.6.1.4.1.5951.4.1.1.23.5 - This represents the time since the NetScaler underwent a state change from primary to secondary or vice-versa
# haTotStateFail (1.3.6.1.4.1.5951.4.1.1.23.6 - Number of times state changed to PARTIAL_FAIL/PARTIAL_FAIL_SSL/ROUTEMONITOR_FAIL/COMPLETE_FAIL.
# haErrSyncFailure (1.3.6.1.4.1.5951.4.1.1.23.7 - Number of times the configuration of the primary and secondary nodes failed to synchronize since that last transition. A synchronization failure results in mismatched configuration. It can be caused by a mismatch in the Remote Procedural Call (RPC) password on the two nodes forming the high availability pair.
# haErrTotNodeDown (1.3.6.1.4.1.5951.4.1.1.23.8 - Total number of heart-beats missed while the peer node was DOWN.
# haErrPropMemFail (1.3.6.1.4.1.5951.4.1.1.23.9 - Total number of times memory allocation failed during command propagation.
# haErrNsbMemFail (1.3.6.1.4.1.5951.4.1.1.23.10 - Total number of times memory allocation failed while sending heartbeats.
# haErrPortSilent (1.3.6.1.4.1.5951.4.1.1.23.11 - Total number of times heartbeat packets were not received on any enabled interface for the duration of the Dead Interval.
# haTotTimerRecoveries (1.3.6.1.4.1.5951.4.1.1.23.12 - Total number of times HA engine recovered from tight loops. (i.e., Total number of times HA timers are not called for MAX down time).
# haNicsMonitorFailed (1.3.6.1.4.1.5951.4.1.1.23.14 - Interfaces on which HA heartbeats are not being seen
# haLastMasterStateTransitionReason (1.3.6.1.4.1.5951.4.1.1.23.15 - The reason for the last master state transition. This gives the conditions under which this node assumed the current state. The current state is available at the oid sysHighAvailabilityMode.0
# haPeerSystemState (1.3.6.1.4.1.5951.4.1.1.23.16 - HA peer system state
# haErrMasterDispute (1.3.6.1.4.1.5951.4.1.1.23.20 - Number of HA master disputes.
# haCurStatus (1.3.6.1.4.1.5951.4.1.1.23.23 - Whether a NetScaler appliance is configured for high availability. Possible values are YES and NO. If the value is NO, the high availability statistics below are invalid.
# haCurState (1.3.6.1.4.1.5951.4.1.1.23.24 - State of the HA node, based on its health, in a high availability setup. Possible values are: UP  Indicates that the node is accessible and can function as either a primary or secondary node. DISABLED  Indicates that the high availability status of the node has been manually disabled. Synchronization and propagation cannot take place between the peer nodes. INIT  Indicates that the node is in the process of becoming part of the high availability configuration. PARTIALFAIL  Indicates that one of the high availability monitored interfaces has failed because of a card or link failure. This state triggers a failover. COMPLETEFAIL  Indicates that all the interfaces of the node are unusable, because the interfaces on which high availability monitoring is enabled are not connected or are manually disabled. This state triggers a failover. DUMB  Indicates that the node is in listening mode. It does not participate in high availability transitions or transfer configuration from the peer node. This is a configured value, not a statistic. PARTIALFAILSSL  Indicates that the SSL card has failed. This state triggers a failover. ROUTEMONITORFAIL  Indicates that the route monitor has failed. This state triggers a failover.

check_cluster_status() {

  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR
  [ -z "$HOSTADDR_1" ] || [ -z "$HOSTADDR_2" ]  && echo "You must specify a server address or host name (-H) for two hosts." && exit $STATE_UNKNOWN

  HOSTNAME_TMP=($(host $HOSTADDR_1))
  HOSTNAME_1=${HOSTNAME_TMP[${#HOSTNAME_TMP[@]} - 1]}
  HOSTNAME_TMP=($(host $HOSTADDR_2))
  HOSTNAME_2=${HOSTNAME_TMP[${#HOSTNAME_TMP[@]} - 1]}
  
  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 $HOSTADDR_2 -m 2 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then

      SNMP_OUT_1=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.23.23.0 .1.3.6.1.4.1.5951.4.1.1.23.24.0 .1.3.6.1.4.1.5951.4.1.1.6.0)
      SNMP_OUT_2=$($SNMPGET -v1 -O vq $HOSTADDR_2 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.23.23.0 .1.3.6.1.4.1.5951.4.1.1.23.24.0 .1.3.6.1.4.1.5951.4.1.1.6.0)

      read HA_CUR_CFG_STATUS_1 HA_CUR_STATUS_1 HA_MODE_1 <<< $SNMP_OUT_1
      read HA_CUR_CFG_STATUS_2 HA_CUR_STATUS_2 HA_MODE_2 <<< $SNMP_OUT_2

      if [ "$HA_CUR_CFG_STATUS_1" == "yes" ] && [ "$HA_CUR_CFG_STATUS_2" == "yes" ] && [ "$HA_CUR_STATUS_1" == "up" ] && [ "$HA_CUR_STATUS_2" == "up" ]
        then

          RETSTR="Cluster-Status: OK<br />$HOSTNAME_1: <b>$HA_CUR_STATUS_1</b> ($HA_MODE_1)<br />$HOSTNAME_2: <b>$HA_CUR_STATUS_2</b> ($HA_MODE_2)"
          RETVAL=$STATE_OK


      elif [ "$HA_CUR_CFG_STATUS_1" == "yes" ] && [ "$HA_CUR_CFG_STATUS_2" == "yes" ]
        then

        if [ "$HA_CUR_STATUS_1" != "up" ] || [ "$HA_CUR_STATUS_2" != "up" ]
          then
            
            RETSTR="Cluster-Status: <b>CRITICAL</b> (high availability status)<br />$HOSTNAME_1: <b>$HA_CUR_STATUS_1</b> ($HA_MODE_1)<br />$HOSTNAME_2: <b>$HA_CUR_STATUS_2</b> ($HA_MODE_2)"
            RETVAL=$STATE_CRITICAL
        fi

      elif [ "$HA_CUR_CFG_STATUS_1" != "yes" ] || [ "$HA_CUR_CFG_STATUS_2" != "yes" ]
          then
            RETSTR="Cluster-Status: <b>CRITICAL</b> (high availability disabled)<br />$HOSTNAME_1 HA enabled: <b>$HA_CUR_CFG_STATUS_1</b><br />$HOSTNAME_2 HA enabled: <b>$HA_CUR_CFG_STATUS_2</b>"
            RETVAL=$STATE_CRITICAL

      fi


    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT"
      RETVAL=STATE_UNKNOWN
  fi

}


check_ha_statistics() {

  read HOSTADDR_1 HOSTADDR_2 <<< $HOSTADDR

  ICMP_RET=$(${NAGIOS_PATH}/libexec/check_icmp -H $HOSTADDR_1 -w 1000.000ms,40% -c 1000.000ms,100%)
  ICMP_STATUS=$(echo $?)
  ICMP_OUT=$(echo $ICMP_RET|cut -d "|" -f1)
  ICMP_PERFDATA=$(echo $ICMP_RET|cut -d "|" -f2)

  if [ "$ICMP_STATUS" == "$STATE_OK" ]
    then

      LAST_STATE_TRANSITION=$($SNMPGET -v1 -O v $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.23.5.0)
      ((SEC=$(echo $LAST_STATE_TRANSITION |cut -d "(" -f2|cut -d ")" -f1)/100))
      ((LST_TIMESTAMP=$(date +%s)-$SEC))
      LST_DATE=$(date -d @$LST_TIMESTAMP +%c)
      LAST_STATE_TRANSITION=$(echo $LAST_STATE_TRANSITION|cut -d")" -f2|cut -d "." -f1)
      TRANSITION_REASON=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.23.15.0)
      SNMP_OUT=$($SNMPGET -v1 -O vq $HOSTADDR_1 -c $COMMUNITY .1.3.6.1.4.1.5951.4.1.1.23.4.0 .1.3.6.1.4.1.5951.4.1.1.23.6.0 .1.3.6.1.4.1.5951.4.1.1.23.7.0 .1.3.6.1.4.1.5951.4.1.1.23.8.0 .1.3.6.1.4.1.5951.4.1.1.23.11.0 .1.3.6.1.4.1.5951.4.1.1.23.21.0 .1.3.6.1.4.1.5951.4.1.1.23.22.0|tr -d '"')

      read TOT_STATE_TRANSITION TOT_STATE_FAIL ERR_SYNC_FAILURE ERR_TOT_NODE_DOWN ERR_PORT_SILENT TOT_PKT_TX TOT_PKT_RX <<< $SNMP_OUT

      PERFDATA="|haTotStateTransitions=${TOT_STATE_TRANSITION}c;;;; haTotStateFail=${TOT_STATE_FAIL}c;;;; haErrSyncFailure=${ERR_SYNC_FAILURE}c;;;; haErrTotNodeDown=${ERR_TOT_NODE_DOWN}c;;;; haErrPortSilent=${ERR_PORT_SILENT}c;;;; haTotPktTx=${TOT_PKT_TX}c;;;; haTotPktRx=${TOT_PKT_RX}c;;;;"
      RETSTR="<p>last_state_transition: $LAST_STATE_TRANSITION ($LST_DATE)<br /></p>transition_reason: $TRANSITION_REASON<br />Total number of master state changes: $TOT_STATE_TRANSITION ${PERFDATA}"
      RETVAL=$STATE_OK

    elif [ "$ICMP_STATUS" == "$STATE_WARNING" ]
      then
        RETSTR="<p>PING: $ICMP_OUT<br /></p>"
        RETVAL=$STATE_WARNING

    elif [ "$ICMP_STATUS" == "$STATE_CRITICAL" ]
      then
        RETSTR="PING: $ICMP_OUT"
        RETVAL=$STATE_CRITICAL

    else
      RETSTR="PING: $ICMP_OUT"
      RETVAL=STATE_UNKNOWN
  fi

}


#! SNMPWALK=$(type -p snmpwalk) && echo "ERROR: MISSING snmpwalk (part of net-snmp)" &&  exit $STATE_UNKNOWN
#! SNMPGET=$(type -p snmpget) && echo "ERROR: MISSING snmpget (part of net-snmp)" &&  exit $STATE_UNKNOWN
#! CHECK_ICMP=$(type -p ${NAGIOS_PATH}/libexec/check_icmp) && echo "ERROR: MISSING ${NAGIOS_PATH}/libexec/check_icmp (part of nagios-plugins)" &&  exit $STATE_UNKNOWN


if [ $# -lt 2 ] && [[ "$1" != "--version" ]] && [[ "$1" != "-V" ]]
  then
    echo "Can not find any argument"
    print_usage
    exit $STATE_UNKNOWN
fi



while test -n "$1"; do
    case "$1" in
        --help)
            print_help
            exit $STATE_OK
            ;;
        -h)
            print_help
            exit $STATE_OK
            ;;
        --version)
            print_head
            exit $STATE_OK
            ;;
        -H)
            grep -q "^-" <<< $2 && echo "Value \"$2\" - is an invalid hostname or address." && print_usage && exit $STATE_UNKNOWN
            if [ -z "$HOSTADDR" ]
              then
                HOSTADDR="$2"
              else
                HOSTADDR="$HOSTADDR $2"
            fi
            shift
            ;;
        -C)
            COMMUNITY=$2
            shift
            ;;
        -e)
            if [ ! -z "$EXEC_COMMAND" ]
              then
                echo "Only one option \"-e\" allowed." && print_usage && exit $STATE_UNKNOWN

            elif [ "$2" == "system_info" ]
              then
                EXEC_COMMAND="check_${2}"

            elif [ "$2" == "cluster_status" ]
              then
                EXEC_COMMAND="check_${2}"

            elif [ "$2" == "interface" ]
              then
                EXEC_COMMAND="check_${2}"
                IF_NAME_CMD=$3
                shift

            elif [ "$2" == "session_stats" ]
              then
                EXEC_COMMAND="check_${2}"

            elif [ "$2" == "hw_health" ]
              then
                EXEC_COMMAND="check_${2}"

            elif [ "$2" == "sys_usage" ]
              then
                EXEC_COMMAND="check_${2}"

            elif [ "$2" == "ha_statistics" ]
              then
                EXEC_COMMAND="check_${2}"

            else
              echo -e "\nUnkown check \"$2\"\n\n" && print_usage && exit $STATE_UNKNOWN
            fi
            shift
            ;;
           *)
               echo "Unknown argument: $1"
               print_usage
               exit $STATE_UNKNOWN
               ;;
       esac
    shift
done


[ -z "$HOSTADDR" ] && echo "You must specify a server address or host name (-H)" && exit $STATE_UNKNOWN
[ -z "$EXEC_COMMAND" ] && echo "You must specify a check (-e)" && exit $STATE_UNKNOWN
[ -z "$COMMUNITY" ] && echo "You must specify snmp community string (-C)" && exit $STATE_UNKNOWN



$EXEC_COMMAND



echo $RETSTR

exit $RETVAL
